@@ -1,5 +1,9 @@
 
 #include <src/Channel.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
+#include <gee/collection.h>
+#include <gee/arraylist.h>
 #include "src/Schedule.h"
 
 
@@ -11,8 +15,9 @@ struct _DVBChannelPrivate {
 	char* _Network;
 	guint* _LogicalChannelNumber;
 	guint _VideoPID;
-	guint _AudioPID;
+	GeeList* _AudioPIDs;
 	guint _Frequency;
+	gboolean _Scrambled;
 	DVBSchedule* schedule;
 	guint sid;
 };
@@ -26,8 +31,9 @@ enum  {
 	DVB_CHANNEL_NETWORK,
 	DVB_CHANNEL_LOGICAL_CHANNEL_NUMBER,
 	DVB_CHANNEL_VIDEO_PID,
-	DVB_CHANNEL_AUDIO_PID,
+	DVB_CHANNEL_AUDIO_PI_DS,
 	DVB_CHANNEL_FREQUENCY,
+	DVB_CHANNEL_SCRAMBLED,
 	DVB_CHANNEL_SCHEDULE
 };
 static gboolean dvb_channel_real_is_valid (DVBChannel* self);
@@ -35,24 +41,74 @@ static gboolean dvb_channel_real_on_same_transport_stream (DVBChannel* self, DVB
 static void dvb_channel_real_setup_dvb_source (DVBChannel* self, GstElement* source);
 static char* dvb_channel_real_to_string (DVBChannel* self);
 static guint* _uint_dup (guint* self);
+static GObject * dvb_channel_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
 static gpointer dvb_channel_parent_class = NULL;
 static void dvb_channel_finalize (GObject* obj);
 
 
 
+char* dvb_channel_get_audio_pids_string (DVBChannel* self) {
+	GString* apids;
+	gint i;
+	const char* _tmp2;
+	char* _tmp3;
+	g_return_val_if_fail (self != NULL, NULL);
+	apids = g_string_new ("");
+	i = 1;
+	{
+		GeeIterator* _pid_it;
+		_pid_it = gee_iterable_iterator ((GeeIterable*) self->priv->_AudioPIDs);
+		while (gee_iterator_next (_pid_it)) {
+			guint pid;
+			pid = GPOINTER_TO_UINT (gee_iterator_get (_pid_it));
+			if (i == gee_collection_get_size ((GeeCollection*) self->priv->_AudioPIDs)) {
+				char* _tmp0;
+				_tmp0 = NULL;
+				g_string_append (apids, _tmp0 = g_strdup_printf ("%u", pid));
+				_tmp0 = (g_free (_tmp0), NULL);
+			} else {
+				char* _tmp1;
+				_tmp1 = NULL;
+				g_string_append (apids, _tmp1 = g_strdup_printf ("%u,", pid));
+				_tmp1 = (g_free (_tmp1), NULL);
+			}
+			i++;
+		}
+		(_pid_it == NULL) ? NULL : (_pid_it = (g_object_unref (_pid_it), NULL));
+	}
+	_tmp2 = NULL;
+	_tmp3 = NULL;
+	return (_tmp3 = (_tmp2 = apids->str, (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), (apids == NULL) ? NULL : (apids = (g_string_free (apids, TRUE), NULL)), _tmp3);
+}
+
+
 static gboolean dvb_channel_real_is_valid (DVBChannel* self) {
 	gboolean _tmp0;
 	gboolean _tmp1;
+	gboolean _tmp2;
 	g_return_val_if_fail (self != NULL, FALSE);
 	_tmp0 = FALSE;
 	_tmp1 = FALSE;
+	_tmp2 = FALSE;
 	if (self->priv->_Name != NULL) {
-		_tmp1 = self->priv->_Frequency != 0;
+		_tmp2 = self->priv->_Frequency != 0;
+	} else {
+		_tmp2 = FALSE;
+	}
+	if (_tmp2) {
+		_tmp1 = dvb_channel_get_Sid (self) != 0;
 	} else {
 		_tmp1 = FALSE;
 	}
 	if (_tmp1) {
-		_tmp0 = dvb_channel_get_Sid (self) != 0;
+		gboolean _tmp3;
+		_tmp3 = FALSE;
+		if (self->priv->_VideoPID != 0) {
+			_tmp3 = TRUE;
+		} else {
+			_tmp3 = gee_collection_get_size ((GeeCollection*) self->priv->_AudioPIDs) != 0;
+		}
+		_tmp0 = (_tmp3);
 	} else {
 		_tmp0 = FALSE;
 	}
@@ -183,7 +239,10 @@ guint* dvb_channel_get_LogicalChannelNumber (DVBChannel* self) {
 
 
 static guint* _uint_dup (guint* self) {
-	return g_memdup (self, sizeof (guint));
+	guint* dup;
+	dup = g_new0 (guint, 1);
+	memcpy (dup, self, sizeof (guint));
+	return dup;
 }
 
 
@@ -209,16 +268,20 @@ void dvb_channel_set_VideoPID (DVBChannel* self, guint value) {
 }
 
 
-guint dvb_channel_get_AudioPID (DVBChannel* self) {
-	g_return_val_if_fail (self != NULL, 0U);
-	return self->priv->_AudioPID;
+GeeList* dvb_channel_get_AudioPIDs (DVBChannel* self) {
+	g_return_val_if_fail (self != NULL, NULL);
+	return self->priv->_AudioPIDs;
 }
 
 
-void dvb_channel_set_AudioPID (DVBChannel* self, guint value) {
+void dvb_channel_set_AudioPIDs (DVBChannel* self, GeeList* value) {
+	GeeList* _tmp2;
+	GeeList* _tmp1;
 	g_return_if_fail (self != NULL);
-	self->priv->_AudioPID = value;
-	g_object_notify ((GObject *) self, "AudioPID");
+	_tmp2 = NULL;
+	_tmp1 = NULL;
+	self->priv->_AudioPIDs = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_AudioPIDs == NULL) ? NULL : (self->priv->_AudioPIDs = (g_object_unref (self->priv->_AudioPIDs), NULL)), _tmp2);
+	g_object_notify ((GObject *) self, "AudioPIDs");
 }
 
 
@@ -235,14 +298,47 @@ void dvb_channel_set_Frequency (DVBChannel* self, guint value) {
 }
 
 
+gboolean dvb_channel_get_Scrambled (DVBChannel* self) {
+	g_return_val_if_fail (self != NULL, FALSE);
+	return self->priv->_Scrambled;
+}
+
+
+void dvb_channel_set_Scrambled (DVBChannel* self, gboolean value) {
+	g_return_if_fail (self != NULL);
+	self->priv->_Scrambled = value;
+	g_object_notify ((GObject *) self, "Scrambled");
+}
+
+
 DVBSchedule* dvb_channel_get_Schedule (DVBChannel* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->schedule;
 }
 
 
+static GObject * dvb_channel_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
+	GObject * obj;
+	DVBChannelClass * klass;
+	GObjectClass * parent_class;
+	DVBChannel * self;
+	klass = DVB_CHANNEL_CLASS (g_type_class_peek (DVB_TYPE_CHANNEL));
+	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
+	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
+	self = DVB_CHANNEL (obj);
+	{
+		GeeArrayList* _tmp0;
+		_tmp0 = NULL;
+		dvb_channel_set_AudioPIDs (self, (GeeList*) (_tmp0 = gee_array_list_new (G_TYPE_UINT, NULL, NULL, g_direct_equal)));
+		(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
+	}
+	return obj;
+}
+
+
 static void dvb_channel_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	DVBChannel * self;
+	gpointer boxed;
 	self = DVB_CHANNEL (object);
 	switch (property_id) {
 		case DVB_CHANNEL_SID:
@@ -263,12 +359,15 @@ static void dvb_channel_get_property (GObject * object, guint property_id, GValu
 		case DVB_CHANNEL_VIDEO_PID:
 		g_value_set_uint (value, dvb_channel_get_VideoPID (self));
 		break;
-		case DVB_CHANNEL_AUDIO_PID:
-		g_value_set_uint (value, dvb_channel_get_AudioPID (self));
+		case DVB_CHANNEL_AUDIO_PI_DS:
+		g_value_set_object (value, dvb_channel_get_AudioPIDs (self));
 		break;
 		case DVB_CHANNEL_FREQUENCY:
 		g_value_set_uint (value, dvb_channel_get_Frequency (self));
 		break;
+		case DVB_CHANNEL_SCRAMBLED:
+		g_value_set_boolean (value, dvb_channel_get_Scrambled (self));
+		break;
 		case DVB_CHANNEL_SCHEDULE:
 		g_value_set_object (value, dvb_channel_get_Schedule (self));
 		break;
@@ -301,12 +400,15 @@ static void dvb_channel_set_property (GObject * object, guint property_id, const
 		case DVB_CHANNEL_VIDEO_PID:
 		dvb_channel_set_VideoPID (self, g_value_get_uint (value));
 		break;
-		case DVB_CHANNEL_AUDIO_PID:
-		dvb_channel_set_AudioPID (self, g_value_get_uint (value));
+		case DVB_CHANNEL_AUDIO_PI_DS:
+		dvb_channel_set_AudioPIDs (self, g_value_get_object (value));
 		break;
 		case DVB_CHANNEL_FREQUENCY:
 		dvb_channel_set_Frequency (self, g_value_get_uint (value));
 		break;
+		case DVB_CHANNEL_SCRAMBLED:
+		dvb_channel_set_Scrambled (self, g_value_get_boolean (value));
+		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -319,6 +421,7 @@ static void dvb_channel_class_init (DVBChannelClass * klass) {
 	g_type_class_add_private (klass, sizeof (DVBChannelPrivate));
 	G_OBJECT_CLASS (klass)->get_property = dvb_channel_get_property;
 	G_OBJECT_CLASS (klass)->set_property = dvb_channel_set_property;
+	G_OBJECT_CLASS (klass)->constructor = dvb_channel_constructor;
 	G_OBJECT_CLASS (klass)->finalize = dvb_channel_finalize;
 	DVB_CHANNEL_CLASS (klass)->is_valid = dvb_channel_real_is_valid;
 	DVB_CHANNEL_CLASS (klass)->on_same_transport_stream = dvb_channel_real_on_same_transport_stream;
@@ -330,8 +433,9 @@ static void dvb_channel_class_init (DVBChannelClass * klass) {
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_NETWORK, g_param_spec_string ("Network", "Network", "Network", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LOGICAL_CHANNEL_NUMBER, g_param_spec_uint ("LogicalChannelNumber", "LogicalChannelNumber", "LogicalChannelNumber", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_VIDEO_PID, g_param_spec_uint ("VideoPID", "VideoPID", "VideoPID", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
-	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_AUDIO_PID, g_param_spec_uint ("AudioPID", "AudioPID", "AudioPID", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_AUDIO_PI_DS, g_param_spec_object ("AudioPIDs", "AudioPIDs", "AudioPIDs", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_FREQUENCY, g_param_spec_uint ("Frequency", "Frequency", "Frequency", 0, G_MAXUINT, 0U, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_SCRAMBLED, g_param_spec_boolean ("Scrambled", "Scrambled", "Scrambled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_SCHEDULE, g_param_spec_object ("Schedule", "Schedule", "Schedule", DVB_TYPE_SCHEDULE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
 }
 
@@ -347,6 +451,7 @@ static void dvb_channel_finalize (GObject* obj) {
 	self->priv->_Name = (g_free (self->priv->_Name), NULL);
 	self->priv->_Network = (g_free (self->priv->_Network), NULL);
 	(self->priv->_LogicalChannelNumber == NULL) ? NULL : (self->priv->_LogicalChannelNumber = (g_free (self->priv->_LogicalChannelNumber), NULL));
+	(self->priv->_AudioPIDs == NULL) ? NULL : (self->priv->_AudioPIDs = (g_object_unref (self->priv->_AudioPIDs), NULL));
 	(self->priv->schedule == NULL) ? NULL : (self->priv->schedule = (g_object_unref (self->priv->schedule), NULL));
 	G_OBJECT_CLASS (dvb_channel_parent_class)->finalize (obj);
 }
