@@ -2,6 +2,7 @@
 #include <src/ChannelListReader.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gee/collection.h>
 #include "src/Utils.h"
 #include "src/Channel.h"
 #include "src/Terrestrial/TerrestrialChannel.h"
@@ -33,6 +34,7 @@ static void dvb_channel_list_reader_set_Type (DVBChannelListReader* self, DVBAda
 static gpointer dvb_channel_list_reader_parent_class = NULL;
 static void dvb_channel_list_reader_finalize (GObject* obj);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 
 
 
@@ -70,7 +72,7 @@ DVBChannelList* dvb_channel_list_reader_read (DVBChannelListReader* self, GError
 	GError * inner_error;
 	char* contents;
 	DVBChannelList* channels;
-	DVBChannelList* _tmp4;
+	DVBChannelList* _tmp3;
 	g_return_val_if_fail (self != NULL, NULL);
 	inner_error = NULL;
 	contents = dvb_utils_read_file_contents (self->priv->_ChannelFile, &inner_error);
@@ -83,29 +85,26 @@ DVBChannelList* dvb_channel_list_reader_read (DVBChannelListReader* self, GError
 		_tmp0 = NULL;
 		return (_tmp0 = NULL, contents = (g_free (contents), NULL), _tmp0);
 	}
-	channels = dvb_channel_list_new ();
+	channels = dvb_channel_list_new (self->priv->_ChannelFile);
 	{
+		char** _tmp1;
 		char** line_collection;
 		int line_collection_length1;
-		char** line_it;
-		line_collection = g_strsplit (contents, "\n", 0);
-		line_collection_length1 = -1;
-		for (line_it = line_collection; *line_it != NULL; line_it = line_it + 1) {
-			const char* _tmp3;
+		int line_it;
+		_tmp1 = NULL;
+		line_collection = _tmp1 = g_strsplit (contents, "\n", 0);
+		line_collection_length1 = _vala_array_length (_tmp1);
+		for (line_it = 0; line_it < _vala_array_length (_tmp1); line_it = line_it + 1) {
+			const char* _tmp2;
 			char* line;
-			_tmp3 = NULL;
-			line = (_tmp3 = *line_it, (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
+			_tmp2 = NULL;
+			line = (_tmp2 = line_collection[line_it], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
 			{
 				if (strlen (line) > 0) {
 					DVBChannel* c;
 					c = dvb_channel_list_reader_parse_line (self, line);
 					if (c != NULL) {
-						DVBChannel* _tmp2;
-						DVBChannel* _tmp1;
-						_tmp2 = NULL;
-						_tmp1 = NULL;
-						dvb_channel_list_add (channels, _tmp2 = (_tmp1 = c, c = NULL, _tmp1));
-						(_tmp2 == NULL) ? NULL : (_tmp2 = (g_object_unref (_tmp2), NULL));
+						dvb_channel_list_add (channels, c);
 					} else {
 						g_warning ("ChannelListReader.vala:27: Could not parse channel");
 					}
@@ -116,13 +115,16 @@ DVBChannelList* dvb_channel_list_reader_read (DVBChannelListReader* self, GError
 		}
 		line_collection = (_vala_array_free (line_collection, line_collection_length1, (GDestroyNotify) g_free), NULL);
 	}
-	_tmp4 = NULL;
-	return (_tmp4 = channels, contents = (g_free (contents), NULL), _tmp4);
+	_tmp3 = NULL;
+	return (_tmp3 = channels, contents = (g_free (contents), NULL), _tmp3);
 }
 
 
 static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* self, const char* line) {
 	DVBChannel* c;
+	DVBChannel* _tmp3;
+	DVBChannel* _tmp4;
+	DVBChannel* _tmp5;
 	g_return_val_if_fail (self != NULL, NULL);
 	g_return_val_if_fail (line != NULL, NULL);
 	c = NULL;
@@ -154,7 +156,15 @@ static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* sel
 			break;
 		}
 	}
-	return c;
+	_tmp3 = NULL;
+	if ((dvb_channel_is_valid (c))) {
+		_tmp3 = c;
+	} else {
+		_tmp3 = NULL;
+	}
+	_tmp4 = NULL;
+	_tmp5 = NULL;
+	return (_tmp5 = (_tmp4 = _tmp3, (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4)), (c == NULL) ? NULL : (c = (g_object_unref (c), NULL)), _tmp5);
 }
 
 
@@ -167,28 +177,31 @@ static DVBChannel* dvb_channel_list_reader_parse_line (DVBChannelListReader* sel
          */
 static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel (const char* line) {
 	DVBTerrestrialChannel* channel;
-	char** _tmp0;
+	char** _tmp1;
+	gint fields_size;
 	gint fields_length1;
+	char** _tmp0;
 	char** fields;
 	gint i;
 	char* val;
-	char* _tmp2;
-	const char* _tmp1;
-	DVBTerrestrialChannel* _tmp3;
+	char* _tmp3;
+	const char* _tmp2;
+	DVBTerrestrialChannel* _tmp4;
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_terrestrial_channel_new ();
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	fields = (_tmp0 = g_strsplit (line, ":", 0), fields_length1 = -1, _tmp0);
+	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
 	i = 0;
 	val = NULL;
+	_tmp3 = NULL;
 	_tmp2 = NULL;
-	_tmp1 = NULL;
-	while ((val = (_tmp2 = (_tmp1 = fields[i], (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), val = (g_free (val), NULL), _tmp2)) != NULL) {
+	while ((val = (_tmp3 = (_tmp2 = fields[i], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), val = (g_free (val), NULL), _tmp3)) != NULL) {
 		if (i == 0) {
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:75: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:76: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -223,7 +236,7 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
 													dvb_channel_set_VideoPID ((DVBChannel*) channel, (guint) atoi (val));
 												} else {
 													if (i == 11) {
-														dvb_channel_set_AudioPID ((DVBChannel*) channel, (guint) atoi (val));
+														gee_collection_add ((GeeCollection*) dvb_channel_get_AudioPIDs ((DVBChannel*) channel), GUINT_TO_POINTER ((guint) atoi (val)));
 													} else {
 														if (i == 12) {
 															dvb_channel_set_Sid ((DVBChannel*) channel, (guint) atoi (val));
@@ -242,8 +255,8 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
 		}
 		i++;
 	}
-	_tmp3 = NULL;
-	return (_tmp3 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp3);
+	_tmp4 = NULL;
+	return (_tmp4 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp4);
 }
 
 
@@ -254,28 +267,31 @@ static DVBTerrestrialChannel* dvb_channel_list_reader_parse_terrestrial_channel
          */
 static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (const char* line) {
 	DVBSatelliteChannel* channel;
-	char** _tmp0;
+	char** _tmp1;
+	gint fields_size;
 	gint fields_length1;
+	char** _tmp0;
 	char** fields;
 	gint i;
 	char* val;
-	char* _tmp2;
-	const char* _tmp1;
-	DVBSatelliteChannel* _tmp3;
+	char* _tmp3;
+	const char* _tmp2;
+	DVBSatelliteChannel* _tmp4;
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_satellite_channel_new ();
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	fields = (_tmp0 = g_strsplit (line, ":", 0), fields_length1 = -1, _tmp0);
+	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
 	i = 0;
 	val = NULL;
+	_tmp3 = NULL;
 	_tmp2 = NULL;
-	_tmp1 = NULL;
-	while ((val = (_tmp2 = (_tmp1 = fields[i], (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), val = (g_free (val), NULL), _tmp2)) != NULL) {
+	while ((val = (_tmp3 = (_tmp2 = fields[i], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), val = (g_free (val), NULL), _tmp3)) != NULL) {
 		if (i == 0) {
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:136: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:137: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -298,7 +314,7 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
 								dvb_channel_set_VideoPID ((DVBChannel*) channel, (guint) atoi (val));
 							} else {
 								if (i == 6) {
-									dvb_channel_set_AudioPID ((DVBChannel*) channel, (guint) atoi (val));
+									gee_collection_add ((GeeCollection*) dvb_channel_get_AudioPIDs ((DVBChannel*) channel), GUINT_TO_POINTER ((guint) atoi (val)));
 								} else {
 									if (i == 7) {
 										dvb_channel_set_Sid ((DVBChannel*) channel, (guint) atoi (val));
@@ -312,8 +328,8 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
 		}
 		i++;
 	}
-	_tmp3 = NULL;
-	return (_tmp3 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp3);
+	_tmp4 = NULL;
+	return (_tmp4 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp4);
 }
 
 
@@ -324,28 +340,31 @@ static DVBSatelliteChannel* dvb_channel_list_reader_parse_satellite_channel (con
          */
 static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char* line) {
 	DVBCableChannel* channel;
-	char** _tmp0;
+	char** _tmp1;
+	gint fields_size;
 	gint fields_length1;
+	char** _tmp0;
 	char** fields;
 	gint i;
 	char* val;
-	char* _tmp2;
-	const char* _tmp1;
-	DVBCableChannel* _tmp3;
+	char* _tmp3;
+	const char* _tmp2;
+	DVBCableChannel* _tmp4;
 	g_return_val_if_fail (line != NULL, NULL);
 	channel = dvb_cable_channel_new ();
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	fields = (_tmp0 = g_strsplit (line, ":", 0), fields_length1 = -1, _tmp0);
+	fields = (_tmp1 = _tmp0 = g_strsplit (line, ":", 0), fields_length1 = _vala_array_length (_tmp0), fields_size = fields_length1, _tmp1);
 	i = 0;
 	val = NULL;
+	_tmp3 = NULL;
 	_tmp2 = NULL;
-	_tmp1 = NULL;
-	while ((val = (_tmp2 = (_tmp1 = fields[i], (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), val = (g_free (val), NULL), _tmp2)) != NULL) {
+	while ((val = (_tmp3 = (_tmp2 = fields[i], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), val = (g_free (val), NULL), _tmp3)) != NULL) {
 		if (i == 0) {
 			if (g_utf8_validate (val, -1, NULL)) {
 				dvb_channel_set_Name ((DVBChannel*) channel, val);
 			} else {
-				g_warning ("ChannelListReader.vala:181: Bad UTF-8 encoded channel name");
+				g_warning ("ChannelListReader.vala:182: Bad UTF-8 encoded channel name");
 				dvb_channel_set_Name ((DVBChannel*) channel, "Bad encoding");
 			}
 		} else {
@@ -368,7 +387,7 @@ static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char*
 									dvb_channel_set_VideoPID ((DVBChannel*) channel, (guint) atoi (val));
 								} else {
 									if (i == 7) {
-										dvb_channel_set_AudioPID ((DVBChannel*) channel, (guint) atoi (val));
+										gee_collection_add ((GeeCollection*) dvb_channel_get_AudioPIDs ((DVBChannel*) channel), GUINT_TO_POINTER ((guint) atoi (val)));
 									} else {
 										if (i == 8) {
 											dvb_channel_set_Sid ((DVBChannel*) channel, (guint) atoi (val));
@@ -383,8 +402,8 @@ static DVBCableChannel* dvb_channel_list_reader_parse_cable_channel (const char*
 		}
 		i++;
 	}
-	_tmp3 = NULL;
-	return (_tmp3 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp3);
+	_tmp4 = NULL;
+	return (_tmp4 = channel, fields = (_vala_array_free (fields, fields_length1, (GDestroyNotify) g_free), NULL), val = (g_free (val), NULL), _tmp4);
 }
 
 
@@ -430,6 +449,7 @@ static void dvb_channel_list_reader_set_Type (DVBChannelListReader* self, DVBAda
 
 static void dvb_channel_list_reader_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
 	DVBChannelListReader * self;
+	gpointer boxed;
 	self = DVB_CHANNEL_LIST_READER (object);
 	switch (property_id) {
 		case DVB_CHANNEL_LIST_READER_CHANNEL_FILE:
@@ -499,19 +519,27 @@ GType dvb_channel_list_reader_get_type (void) {
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) array)[i] != NULL)
-			destroy_func (((gpointer*) array)[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) array)[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 
 
