@@ -41,7 +41,6 @@ struct _DVBIDBusTerrestrialScannerIface {
 	void (*Run) (DVBIDBusTerrestrialScanner* self);
 	void (*Destroy) (DVBIDBusTerrestrialScanner* self);
 	gboolean (*WriteChannelsToFile) (DVBIDBusTerrestrialScanner* self, const char* path);
-	guint (*GetQueueSize) (DVBIDBusTerrestrialScanner* self);
 	void (*AddScanningData) (DVBIDBusTerrestrialScanner* self, guint frequency, guint hierarchy, guint bandwidth, const char* transmode, const char* code_rate_hp, const char* code_rate_lp, const char* constellation, guint guard);
 	gboolean (*AddScanningDataFromFile) (DVBIDBusTerrestrialScanner* self, const char* path);
 };
@@ -59,7 +58,6 @@ struct _DVBTerrestrialScannerClass {
 void dvb_id_bus_terrestrial_scanner_Run (DVBIDBusTerrestrialScanner* self);
 void dvb_id_bus_terrestrial_scanner_Destroy (DVBIDBusTerrestrialScanner* self);
 gboolean dvb_id_bus_terrestrial_scanner_WriteChannelsToFile (DVBIDBusTerrestrialScanner* self, const char* path);
-guint dvb_id_bus_terrestrial_scanner_GetQueueSize (DVBIDBusTerrestrialScanner* self);
 void dvb_id_bus_terrestrial_scanner_AddScanningData (DVBIDBusTerrestrialScanner* self, guint frequency, guint hierarchy, guint bandwidth, const char* transmode, const char* code_rate_hp, const char* code_rate_lp, const char* constellation, guint guard);
 gboolean dvb_id_bus_terrestrial_scanner_AddScanningDataFromFile (DVBIDBusTerrestrialScanner* self, const char* path);
 void dvb_id_bus_terrestrial_scanner_dbus_register_object (DBusConnection* connection, const char* path, void* object);
