@@ -31,6 +31,7 @@ enum  {
 	DVB_TIMER_MINUTE,
 	DVB_TIMER_DURATION
 };
+static DVBOverlapType dvb_timer_get_overlap (const time_t* this_start, const time_t* this_end, const time_t* other_start, const time_t* other_end);
 static time_t dvb_timer_get_end_time_timestamp (DVBTimer* self);
 static time_t dvb_timer_get_start_time_timestamp (DVBTimer* self);
 static void dvb_timer_set_Id (DVBTimer* self, guint32 value);
@@ -130,27 +131,75 @@ gboolean dvb_timer_conflicts_with (DVBTimer* self, DVBTimer* t2) {
 
 
 /**
-         * Whether the timer is scheduled somewhere in the given timeframe
+         * @duration: in minutes
+         * @returns: The overlap between the timer and the given time range.
+         * The timer is the reference, i.e. if the time range is completely
+         * contained in the timer OverlapType.COMPLETE is returned.
+         *
+         * The given time range must be in local time.
          */
-gboolean dvb_timer_is_in_range (DVBTimer* self, guint start_year, guint start_month, guint start_day, guint start_hour, guint start_minute, guint duration) {
-	gint64 this_start;
+DVBOverlapType dvb_timer_get_overlap_local (DVBTimer* self, guint start_year, guint start_month, guint start_day, guint start_hour, guint start_minute, guint duration) {
+	time_t this_start;
+	time_t this_end;
 	struct tm other_time;
 	time_t other_start;
-	g_return_val_if_fail (self != NULL, FALSE);
-	this_start = (gint64) dvb_timer_get_start_time_timestamp (self);
+	time_t other_end;
+	g_return_val_if_fail (self != NULL, 0);
+	this_start = dvb_timer_get_start_time_timestamp (self);
+	this_end = dvb_timer_get_end_time_timestamp (self);
 	other_time = dvb_utils_create_time ((gint) start_year, (gint) start_month, (gint) start_day, (gint) start_hour, (gint) start_minute, 0);
 	other_start = mktime (&other_time);
-	if (this_start <= other_start) {
-		time_t this_end;
+	other_time.tm_min = other_time.tm_min + ((gint) duration);
+	other_end = mktime (&other_time);
+	return dvb_timer_get_overlap (&this_start, &this_end, &other_start, &other_end);
+}
+
+
+/**
+         * Same as get_overlap_local but the given time range is UTC time.
+         */
+DVBOverlapType dvb_timer_get_overlap_utc (DVBTimer* self, guint start_year, guint start_month, guint start_day, guint start_hour, guint start_minute, guint duration) {
+	time_t this_start;
+	time_t this_end;
+	struct tm other_time;
+	time_t other_start;
+	time_t other_end;
+	g_return_val_if_fail (self != NULL, 0);
+	this_start = dvb_timer_get_start_time_timestamp (self);
+	this_end = dvb_timer_get_end_time_timestamp (self);
+	other_time = dvb_utils_create_time ((gint) start_year, (gint) start_month, (gint) start_day, (gint) start_hour, (gint) start_minute, 0);
+	other_start = timegm (&other_time);
+	other_time.tm_min = other_time.tm_min + ((gint) duration);
+	other_end = timegm (&other_time);
+	return dvb_timer_get_overlap (&this_start, &this_end, &other_start, &other_end);
+}
+
+
+static DVBOverlapType dvb_timer_get_overlap (const time_t* this_start, const time_t* this_end, const time_t* other_start, const time_t* other_end) {
+	if ((*this_start) <= (*other_start)) {
 		/* No conflict when this timer ends before other starts*/
-		this_end = dvb_timer_get_end_time_timestamp (self);
-		return (this_end > other_start);
+		if ((*this_end) <= (*other_start)) {
+			/* this starts before other and ends before other starts*/
+			return DVB_OVERLAP_TYPE_NONE;
+		} else {
+			if ((*this_end) >= (*other_end)) {
+				return DVB_OVERLAP_TYPE_COMPLETE;
+			} else {
+				return DVB_OVERLAP_TYPE_PARTIAL;
+			}
+		}
 	} else {
-		time_t other_end;
 		/* No conflict when other timer ends before this starts*/
-		other_time.tm_min = other_time.tm_min + ((gint) duration);
-		other_end = mktime (&other_time);
-		return (other_end > this_start);
+		if ((*this_end) <= (*other_end)) {
+			/* this starts after other and ends before other*/
+			return DVB_OVERLAP_TYPE_PARTIAL;
+		} else {
+			if ((*this_start) < (*other_end)) {
+				return DVB_OVERLAP_TYPE_PARTIAL;
+			} else {
+				return DVB_OVERLAP_TYPE_NONE;
+			}
+		}
 	}
 }
 
