@@ -5,20 +5,21 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <time.h>
-#include <gio/gio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
 
 
+#define DVB_UTILS_BUFFER_SIZE 4096
 const char* dvb_utils_get_nick_from_enum (GType enumtype, gint val);
 gint dvb_utils_get_value_by_name_from_enum (GType enumtype, const char* name);
 const char* dvb_utils_get_name_by_value_from_enum (GType enumtype, gint val);
 void dvb_utils_mkdirs (GFile* directory, GError** error);
 char* dvb_utils_remove_nonalphanums (const char* text);
-gint64 dvb_utils_difftime (struct tm* t1, struct tm* t2);
+gint64 dvb_utils_difftime (const struct tm* t1, const struct tm* t2);
 struct tm dvb_utils_create_time (gint year, gint month, gint day, gint hour, gint minute, gint second);
 struct tm dvb_utils_create_utc_time (gint year, gint month, gint day, gint hour, gint minute, gint second);
 char* dvb_utils_read_file_contents (GFile* file, GError** error);
